import path from 'path';
import { homedir } from 'os';
import fs from 'fs';
import { checkFileExists, createDir } from '../file/index.ts';
import { ProxyInfo } from '../proxy/proxy.ts';

/**
 * 助手配置文件路径, 全局配置文件目录
 */
const configDir = createDir(path.join(homedir(), '.config/envision/assistant-app'));

/**
 * 助手配置文件初始化
 * @param configRootPath
 * @returns
 */
export const initConfig = (configRootPath: string) => {
  const configDir = createDir(path.join(configRootPath, 'assistant-app'));
  const configPath = path.join(configDir, 'assistant-config.json');
  const pageConfigPath = path.join(configDir, 'assistant-page-config.json');
  const pageDir = createDir(path.join(configDir, 'page'));
  const appsDir = createDir(path.join(configDir, 'apps'));
  const appsConfigPath = path.join(configDir, 'assistant-apps-config.json');
  const appPidPath = path.join(configDir, 'assistant-app.pid');
  const envConfigPath = path.join(configDir, '.env');
  return {
    /**
     * 助手配置文件路径
     */
    configDir,
    /**
     * 助手配置文件路径, assistant-config.json
     */
    configPath,
    /**
     * 服务目录， 后端服务目录
     */
    appsDir,
    /**
     * 服务配置文件路径 assistant-apps-config.json
     */
    appsConfigPath,
    /**
     * 应用目录， 前端应用目录
     */
    pageDir,
    /**
     * 应用配置文件路径, assistant-page-config.json
     */
    pageConfigPath,
    /**
     * 应用进程pid文件路径
     */
    appPidPath,
    /**
     * 环境变量配置文件路径
     */
    envConfigPath,
  };
};
export type ReturnInitConfigType = ReturnType<typeof initConfig>;

export type AssistantConfigData = {
  pageApi?: string; // https://kevisual.cn
  registry?: string; // https://kevisual.cn
  proxy?: ProxyInfo[];
  apiProxyList?: ProxyInfo[];
  description?: string;
  /**
   * 首页
   */
  home?: string;
  ai?: {
    provider?: string | 'DeepSeek' | 'SiliconFlow';
    apiKey?: string;
    model?: string;
  };
};
let assistantConfig: AssistantConfigData;
type AssistantConfigOptions = {
  configDir?: string;
  init?: boolean;
};
export class AssistantConfig {
  config: AssistantConfigData;
  #configPath: ReturnInitConfigType;
  configDir: string;
  isMountedConfig?: boolean;
  constructor(opts?: AssistantConfigOptions) {
    this.configDir = opts?.configDir || configDir;
    if (opts?.init) {
      this.init();
    }
  }
  set configPath(configPath: ReturnInitConfigType) {
    this.#configPath = configPath;
  }
  get configPath() {
    if (!this.#configPath) {
      initConfig(this.configDir);
    }
    return this.#configPath;
  }
  init() {
    this.configPath = initConfig(this.configDir);
    this.isMountedConfig = true;
  }
  checkMounted() {
    if (!this.isMountedConfig) {
      this.init();
    }
  }
  getConfigPath() {}
  getConfig() {
    try {
      if (!checkFileExists(this.configPath.configPath)) {
        fs.writeFileSync(this.configPath.configPath, JSON.stringify({ proxy: [] }, null, 2));
        return {
          pageApi: '',
          proxy: [],
        };
      }
      assistantConfig = JSON.parse(fs.readFileSync(this.configPath.configPath, 'utf8'));
      return assistantConfig;
    } catch (error) {
      console.error('file read', error.message);
      return {
        pageApi: '',
        proxy: [],
      };
    }
  }
  getCacheAssistantConfig() {
    if (this.config) {
      return this.config;
    }
    return this.getConfig();
  }
  getRegistry() {
    const config = this.getCacheAssistantConfig();
    return config?.registry || config?.pageApi;
  }
  /**
   * 设置 assistant-config.json 配置
   * @param config
   * @returns
   */
  setConfig(config?: AssistantConfigData) {
    const myConfig = this.getCacheAssistantConfig();
    const newConfig = { ...myConfig, ...config };
    this.config = newConfig;
    fs.writeFileSync(this.configPath.configPath, JSON.stringify(newConfig, null, 2));
    return newConfig;
  }
  /**
   * 应用配置
   * @returns
   */
  getPageConfig(): AppConfig {
    const { pageConfigPath } = this.configPath;
    if (!checkFileExists(pageConfigPath)) {
      return {
        list: [],
      };
    }
    return JSON.parse(fs.readFileSync(pageConfigPath, 'utf8'));
  }
  setAppConfig(config?: AppConfig) {
    const _config = this.getPageConfig();
    const _saveConfig = { ..._config, ...config };
    const { pageConfigPath } = this.configPath;

    fs.writeFileSync(pageConfigPath, JSON.stringify(_saveConfig, null, 2));
    return _saveConfig;
  }
  addAppConfig(app: any) {
    const config = this.getPageConfig();
    const assistantConfig = this.getCacheAssistantConfig();
    const _apps = config.list;
    const _proxy = assistantConfig.proxy || [];
    const { user, key } = app;
    const newProxyInfo = {
      user,
      key,
      path: `/${user}/${key}`,
    };
    const _proxyIndex = _proxy.findIndex((_proxy: any) => _proxy.path === newProxyInfo.path);
    if (_proxyIndex !== -1) {
      _proxy[_proxyIndex] = newProxyInfo;
    } else {
      _proxy.push(newProxyInfo);
    }

    const _app = _apps.findIndex((_app: any) => _app.id === app.id);
    if (_app !== -1) {
      _apps[_app] = app;
    } else {
      _apps.push(app);
    }
    this.setAppConfig({ ...config, list: _apps });
    this.setConfig({ ...assistantConfig, proxy: _proxy });
    return config;
  }
  getAppList() {
    return this.getPageConfig().list;
  }
  /**
   * process.env.ASSISTANT_CONFIG_DIR || process.cwd()
   * configDir是助手配置文件目录，文件内部包函
   * assistant-config.json 配置文件
   * assistant-page-config.json 应用配置文件
   * assistant-app.pid 应用进程pid文件
   * .env 环境变量配置文件
   * apps: 服务目录
   * page: 应用目录
   * pem: 证书目录
   * @param configDir
   */
  static detectConfigDir(configDir?: string, deep = 3) {
    const checkConfigDir = path.resolve(configDir || process.env.ASSISTANT_CONFIG_DIR || process.cwd());
    const configPath = path.join(checkConfigDir, 'assistant-app');
    if (checkFileExists(configPath)) {
      return path.join(checkConfigDir);
    }
    if (deep >= 2) {
      const lastConfigPath = path.join(checkConfigDir, '..', 'assistant-app');
      if (checkFileExists(lastConfigPath)) {
        return path.join(checkConfigDir, '..');
      }
    }
    if (deep >= 3) {
      const lastConfigPath2 = path.join(checkConfigDir, '../..', 'assistant-app');
      if (checkFileExists(lastConfigPath2)) {
        return path.join(checkConfigDir, '../..');
      }
    }
    // 如果没有找到助手配置文件目录，则返回当前目录, 执行默认创建助手配置文件目录
    return checkConfigDir;
  }
}

type AppConfig = {
  list: any[];
};
