import type { Server as HttpsServer } from 'node:https';
import type { Server as HttpServer } from 'node:http';
import type { Http2Server } from 'node:http2';
import { WebSocket } from 'ws';
import { ProxyInfo } from './proxy.ts';
import { WebSocketServer } from 'ws';

export const wss = new WebSocketServer({
  noServer: true,
});

type WssAppOptions = {
  wss: WebSocketServer;
  apiList: ProxyInfo[];
};
class WssApp {
  wss: WebSocketServer;
  apiList: ProxyInfo[];
  constructor(opts: WssAppOptions) {
    this.wss = opts.wss;
    this.apiList = opts.apiList;
  }
  upgrade(request: any, socket: any, head: any) {
    const req = request as any;
    const wss = this.wss;
    const url = new URL(req.url, 'http://localhost');
    const id = url.searchParams.get('id');
    console.log('upgrade', request.url, id);
    wss.handleUpgrade(req, socket, head, (ws) => {
      wss.emit('connection', ws, req);
    });
  }
  async handleConnection(ws: WebSocket, req: any) {
    console.log('connected', req.url);
    const that = this;
    const proxyApiList: ProxyInfo[] = that.apiList || [];
    const proxyApi = proxyApiList.find((item) => req.url.startsWith(item.path));

    if (proxyApi && proxyApi.ws) {
      const _u = new URL(req.url, `${proxyApi.target}`);
      if (proxyApi.pathname) {
        _u.pathname = _u.pathname.replace(proxyApi.path, proxyApi.pathname);
      }
      const isHttps = _u.protocol === 'https:';
      const wsProtocol = isHttps ? 'wss' : 'ws';
      const wsUrl = `${wsProtocol}://${_u.host}${_u.pathname}`;
      console.log('WebSocket proxy URL', { wsUrl });
      const proxySocket = new WebSocket(wsUrl, {
        // headers: req.headers,
        rejectUnauthorized: false,
      });
      proxySocket.on('open', () => {
        ws.on('message', (message) => {
          console.log('WebSocket client message', message);
          proxySocket.send(message);
        });
        proxySocket.on('message', (message) => {
          // console.log('WebSocket proxy message', message);
          ws.send(message.toString());
        });
      });

      proxySocket.on('error', (err) => {
        console.error(`WebSocket proxy error: ${err.message}`);
      });
      proxySocket.on('close', () => {
        console.log('WebSocket proxy closed');
      });
      ws.on('error', (err) => {
        console.error('WebSocket client error', err);
        proxySocket?.close?.();
      });
      ws.on('close', () => {
        console.log('WebSocket client closed');
        proxySocket?.close?.();
      });
    } else {
      ws.send(JSON.stringify({ type: 'error', message: 'not found' }));
      ws.close();
    }
  }
}
/**
 * websocket代理
 * apiList: [{ path: '/api/router', target: 'https://kevisual.xiongxiao.me' }]
 * @param server
 * @param config
 */
export const wsProxy = (server: HttpServer | HttpsServer | Http2Server, config: { apiList: ProxyInfo[] }) => {
  // console.log('Upgrade initialization started');
  const wssApp = new WssApp({
    wss: wss,
    apiList: config.apiList,
  });
  wss.on('connection', async (ws, req) => {
    // console.log('WebSocket connection established');
    await wssApp.handleConnection(ws, req);
  });
  // 处理升级请求
  server.on('upgrade', (request, socket, head) => {
    wssApp.upgrade(request, socket, head);
  });
};
